@@ -1,4 +1,4 @@
-# Copyright 2018 Red Hat, Inc.
+# Copyright 2018-2020 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,12 +36,23 @@ SUBNET_ENDPOINT = ENDPOINT + 'subnets/'
 PORT_ENDPOINT = ENDPOINT + 'ports/'
 
 
+EXT_NET_NAME = 'extnet'
+PROVIDER_TYPE = 'vlan'
+VLAN_ID = 666
+
+
 @pytest.fixture(scope='module')
 def logical_switch():
     with _network('ls0') as network:
         yield network
 
 
+@pytest.fixture
+def ext_net_logical_switch():
+    with _network('pls0', localnet=EXT_NET_NAME, vlan_id=VLAN_ID) as network:
+        yield network
+
+
 @pytest.fixture(scope='module')
 def subnet(logical_switch):
     with _subnet(logical_switch) as subnet:
@@ -109,6 +120,37 @@ def broken_port(logical_switch, subnet):
     )
 
 
+@pytest.fixture(
+    scope='module', params=[True, False], ids=['with_subnet', 'without_subnet']
+)
+def broken_port_sec_group(request, logical_switch, subnet):
+    nonexistent_sec_group = '2d6a7cf1-0105-4864-b94b-8ff0b30c41a7'
+    payload = {
+        'port': {
+            'admin_state_up': True,
+            'mac_address': 'fa:16:3e:c9:cb:f0',
+            'name': 'broken-port',
+            'network_id': logical_switch['id'],
+            'port_security_enabled': True,
+            'project_id': 'd6700c0c9ffa4f1cb322cd4a1f3906fa',
+            'security_groups': [
+                nonexistent_sec_group
+            ],
+            'tenant_id': 'd6700c0c9ffa4f1cb322cd4a1f3906fa',
+        }
+    }
+    if request.param:
+        payload['fixed_ips'] = {'subnet_id': subnet['id']}
+
+    response = requests.post(
+        'http://localhost:9696/v2.0/ports/', json=payload
+    )
+    _expect_failure(
+        response, 500,
+        'Port group {} does not exist'.format(nonexistent_sec_group)
+    )
+
+
 @pytest.fixture(scope='module')
 def icmp_group():
     with SecurityGroup('icmp_group', None) as sec_group:
@@ -134,6 +176,18 @@ def test_get_network(logical_switch):
     assert networks.pop()['name'] == logical_switch['name']
 
 
+def test_get_physnet(ext_net_logical_switch):
+    networks = _get_and_assert(
+        'networks', filter_key='name', filter_value='pls0'
+    )
+    assert len(networks) == 1
+    physnet = networks.pop()
+    assert physnet['name'] == ext_net_logical_switch['name']
+    assert physnet['provider:physical_network'] == EXT_NET_NAME
+    assert physnet['provider:network_type'] == PROVIDER_TYPE
+    assert physnet['provider:segmentation_id'] == VLAN_ID
+
+
 def test_filter(logical_switch):
     networks = _get_and_assert('networks', 'name', 'ls0')
     assert len(networks) == 1
@@ -199,19 +253,31 @@ def test_update_network_with_subnet_mtu(logical_switch, subnet):
                           update_network_data)
 
 
-def test_create_invalid_port_no_leftovers(subnet, broken_port):
+def test_create_invalid_port_with_mac(subnet, broken_port):
     json_response = _get_and_assert('ports')
-    assert len(json_response) == 1
     assert json_response[0]['name'] == 'private-port'
-    broken_port_data = _get_and_assert(
-        'ports', filter_key='name', filter_value='broken-port'
-    )
-    assert len(broken_port_data) == 0
+
     private_port_data = _get_and_assert(
         'ports', filter_key='name', filter_value='private-port'
     )
     assert json_response == private_port_data
 
+    _expect_no_leftovers('broken-port')
+
+
+def test_create_port_with_nonexisting_sec_group(
+    broken_port_sec_group
+):
+    json_response = _get_and_assert('ports')
+    assert json_response[0]['name'] == 'private-port'
+
+    private_port_data = _get_and_assert(
+        'ports', filter_key='name', filter_value='private-port'
+    )
+    assert json_response == private_port_data
+
+    _expect_no_leftovers('broken-port')
+
 
 class TestSecurityGroupsApi(object):
     def test_rule_to_group_association(self, icmp_group):
@@ -270,6 +336,13 @@ def test_not_found_escape():
     assert data['error']['message'].startswith('Incorrect path')
 
 
+def _expect_no_leftovers(broken_port_name):
+    broken_port_data = _get_and_assert(
+        'ports', filter_key='name', filter_value=broken_port_name,
+    )
+    assert not broken_port_data
+
+
 def _get_and_assert(entity_type, filter_key=None, filter_value=None):
     url = ENDPOINT + entity_type + (
         '?{key}={value}'.format(key=filter_key, value=filter_value)
@@ -286,9 +359,15 @@ def _expect_failure(response, expected_status_code, expected_error_message):
 
 
 @contextlib.contextmanager
-def _network(name):
+def _network(name, localnet=None, vlan_id=None):
+    network_data = {'name': name}
+    if localnet:
+        network_data['provider:physical_network'] = localnet
+    if vlan_id:
+        network_data['provider:network_type'] = 'vlan'
+        network_data['provider:segmentation_id'] = vlan_id
     payload = {
-        'network': {'name': name}
+        'network': network_data
     }
     response = requests.post(
         'http://localhost:9696/v2.0/networks/', json=payload
